package ch.hslu.appe.order.logic;

import java.util.List;

import ch.hslu.appe.order.logic.exception.OrderException;
import ch.hslu.appe.order.logic.exception.OrderNotExistsException;
import ch.hslu.appe.order.model.Bill;
import ch.hslu.appe.order.model.Order;
import ch.hslu.appe.order.model.OrderState;

/**
 * Order System Interface.
 */
public interface OrderSystem {

    /**
     * Adds a Position to the Order.
     * @param userId The user id who is logged on
     * @param orderNumber The order number
     * @param productNumber The product number
     * @param amount The amount
     * @return Returns Returns the auto-assigned position number on success, otherwise -1.
     * @throws OrderException PositionAlreadyExistsException
     */
    int addPosition(int userId, int orderNumber, int productNumber, int amount) throws OrderException;

    /**
     * Annulates an order. No matter whether the order is stored in the database or not.
     * @param userId The user id who is logged on
     * @param number The order number
     * @return Returns true on success, otherwise false.
     * @throws OrderException OrderException
     */
    boolean annulateOrder(int userId, int number) throws OrderException;

    /**
     * Creates a new order. But it does not persist this created order.
     * @param userId The user id who is logged on
     * @param order The order instance
     * @return Returns the auto-assigned order number on success, otherwise -1.
     * @throws OrderException OrderCreationFailedException
     */
    int createOrder(int userId, Order order) throws OrderException;

    /**
     * Deletes a position by using the the order- and position number.
     * @param userId The user id who is logged on
     * @param orderNumber The order number
     * @param positionNumber The position number
     * @return Returns true on success, otherwise false.
     * @throws OrderException OrderException
     */
    boolean deletePosition(int userId, int orderNumber, int positionNumber) throws OrderException;

    /**
     * Gets the active order from the user.
     * @param userId The user id who is logged on
     * @return Returns the active order if the user has one, otherwise null
     */
    Order getActiveOrder(int userId);

    /**
     * Gets an order by its number.
     * @param userId The user id
     * @param number The order number
     * @return Returns the order if it exists, otherwise null.
     * @throws OrderException OrderException
     */
    Order getOrder(int userId, int number) throws OrderException;

    /**
     * Gets all orders.
     * @return Returns a list of orders
     */
    List<Order> getOrders();

    /**
     * Gets the status of an order.
     * @param orderNumber The order number
     * @return Returns the order state.
     * @throws OrderNotExistsException OrderNotExistsException
     */
    OrderState getStatus(int orderNumber) throws OrderNotExistsException;

    /**
     * Releases the lock of an order.
     * @param userId The user id
     * @param orderNumber The order number
     * @throws OrderException OrderNotCheckedOutException
     */
    void releaseOrder(int userId, int orderNumber) throws OrderException;

    /**
     * Sends a bill to a customer.
     * @param userId The user id who is logged on
     * @param orderNumber The order number
     * @param bill The bill instance
     * @return Returns true on success.
     * @throws OrderException OrderException
     */
    boolean sendBill(int userId, int orderNumber, Bill bill) throws OrderException;

    /**
     * Sets the state of an order.
     * @param userId The user id
     * @param orderNumber The order number
     * @param state The state
     * @return Returns true on success, otherwise false.
     * @throws OrderException OrderException
     */
    boolean setStatus(int userId, int orderNumber, OrderState state) throws OrderException;

    /**
     * Persists an order.
     * @param userId The user id
     * @param number The order number
     * @return Returns true on success, otherwise false
     * @throws OrderException OrderException
     */
    boolean submitOrder(int userId, int number) throws OrderException;

    /**
     * Updates an order.
     * @param userId The user id
     * @param order The order
     * @return Returns true on success, otherwise false
     * @throws OrderException OrderException
     */
    boolean updateOrder(int userId, Order order) throws OrderException;

    /**
     * Updates a position.
     * @param userId The user id
     * @param orderNumber The order number
     * @param positionNumber The position number
     * @param amount The amount
     * @return Returns true on success, otherwise false.
     * @throws OrderException OrderException
     */
    boolean updatePosition(int userId, int orderNumber, int positionNumber, int amount) throws OrderException;
}
