package ch.hslu.appe.order.webservice;

import java.util.List;

import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.jws.soap.SOAPBinding.Style;
import javax.xml.soap.SOAPException;

import ch.hslu.appe.order.logic.exception.OrderException;
import ch.hslu.appe.order.logic.exception.OrderNotExistsException;
import ch.hslu.appe.order.model.OrderState;
import ch.hslu.appe.order.webservice.dto.OrderDto;
import ch.hslu.appe.usermanagement.exception.IllegalNameException;

/**
 * Order Service Interface.
 */
@WebService
@SOAPBinding(style = Style.DOCUMENT)
public interface OrderService {

    /**
     * Adds a Position to the Order.
     * @param orderNumber The order number
     * @param productNumber The product number
     * @param amount The amount
     * @return Returns Returns the auto-assigned position number on success, otherwise -1.
     * @throws OrderException OrderException
     * @throws SOAPException SOAPException
     */
    @WebMethod
    int addPosition(int orderNumber, int productNumber, int amount) throws OrderException, SOAPException;

    /**
     * Annulates an order. No matter whether the order is stored in the database or not.
     * @param number The order number
     * @return Returns true on success, otherwise false.
     * @throws OrderException OrderException
     * @throws SOAPException SOAPException
     */
    @WebMethod
    boolean annulateOrder(int number) throws OrderException, SOAPException;

    /**
     * Creates a new order. But it does not persist this created order.
     * @param order The order instance
     * @return Returns the auto-assigned order number on success, otherwise -1.
     * @throws OrderException OrderException
     * @throws IllegalNameException IllegalNameException
     * @throws SOAPException SOAPException
     */
    @WebMethod
    int createOrder(OrderDto order) throws OrderException, IllegalNameException, SOAPException;

    /**
     * Deletes a position by using the the order- and position number.
     * @param orderNumber The order number
     * @param positionNumber The position number
     * @return Returns true on success, otherwise false.
     * @throws OrderException OrderException
     * @throws SOAPException SOAPException
     */
    @WebMethod
    boolean deletePosition(int orderNumber, int positionNumber) throws OrderException, SOAPException;

    /**
     * Gets the active order from the user.
     * @return Returns the active order if the user has one, otherwise null
     * @throws SOAPException SOAPException
     */
    @WebMethod
    OrderDto getActiveOrder() throws SOAPException;

    /**
     * Gets an order by its number.
     * @param number The order number
     * @return Returns the order if it exists, otherwise null.
     * @throws OrderException OrderException
     * @throws SOAPException SOAPException
     */
    @WebMethod
    OrderDto getOrder(int number) throws OrderException, SOAPException;

    /**
     * Gets all orders.
     * @return Returns a list of orders
     */
    @WebMethod
    List<OrderDto> getOrders();

    /**
     * Gets the status of an order.
     * @param orderNumber The order number
     * @return Returns the order state.
     * @throws OrderNotExistsException OrderNotExistsException
     */
    @WebMethod
    OrderState getStatus(int orderNumber) throws OrderNotExistsException;

    /**
     * Releases the lock of an order.
     * @param orderNumber The order number
     * @throws SOAPException
     * @throws OrderException OrderException
     * @throws SOAPException SOAPException
     */
    @WebMethod
    void releaseOrder(int orderNumber) throws OrderException, SOAPException;

    /**
     * Sets the state of an order.
     * @param orderNumber The order number
     * @param state The state
     * @return Returns true on success, otherwise false.
     * @throws OrderException OrderException
     * @throws SOAPException SOAPException
     */
    @WebMethod
    boolean setStatus(int orderNumber, OrderState state) throws OrderException, SOAPException;

    /**
     * Persists an order.
     * @param number The order number
     * @return Returns true on success, otherwise false
     * @throws OrderException OrderException
     * @throws SOAPException SOAPException
     */
    @WebMethod
    boolean submitOrder(int number) throws OrderException, SOAPException;

    /**
     * Updates an order.
     * @param order The order
     * @return Returns true on success, otherwise false
     * @throws OrderException OrderException
     * @throws IllegalNameException IllegalNameException
     * @throws SOAPException SOAPException
     */
    @WebMethod
    boolean updateOrder(OrderDto order) throws OrderException, IllegalNameException, SOAPException;

    /**
     * Updates a position.
     * @param orderNumber The order number
     * @param positionNumber The position number
     * @param amount The amount
     * @return Returns true on success, otherwise false.
     * @throws OrderException OrderException
     * @throws SOAPException SOAPException
     */
    @WebMethod
    boolean updatePosition(int orderNumber, int positionNumber, int amount) throws OrderException, SOAPException;
}
