package ch.hslu.appe.inventory;

import static org.junit.Assert.assertEquals;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import ch.hslu.appe.inventory.logic.InventorySystem;
import ch.hslu.appe.inventory.logic.InventorySystemImpl;
import ch.hslu.appe.inventory.logic.exception.ReservationNotExistsException;
import ch.hslu.appe.inventory.logic.exception.ReservationNotPossible;
import ch.hslu.appe.inventory.model.Product;

/**
 * Simple TestClass for Inventory.
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public final class InventoryTest {

    // inactive Product 10'000
    private static final boolean PROD10000_ACTIVE = false;
    private static final int PROD10000_NUMBER = 10000;
    // active and valid product 10'001
    private static final boolean PROD10001_ACTIVE = true;
    private static final int PROD10001_AMOUNT = 26;
    private static final int PROD10001_AMOUNTMIN = 3;
    private static final boolean PROD10001_EXISTS = true;
    private static final int PROD10001_ID = 2;
    private static final String PROD10001_NAME = "Product 10001";
    private static final int PROD10001_NUMBER = 10001;
    private static final float PROD10001_PRICE = 45.432f;
    // active and valid product 10'003
    private static final int PROD10003_NUMBER = 10003;
    // active and valid product 10'004
    private static final int PROD10004_NUMBER = 10004;
    // Non existent product 666
    private static final boolean PROD666_EXISTS = false;
    private static final int PROD666_NUMBER = 666;

    private final InventorySystem inventory = new InventorySystemImpl();

    /**
     * JUnit SetUp.
     */
    @Before
    public void setup() {

        Product product = new Product();
        // SetUp Product 10000
        product = inventory.getProduct(PROD10000_NUMBER);
        product.setActive(PROD10000_ACTIVE);
        inventory.updateProduct(product);

        // SetUp Product 10001
        product.setId(PROD10001_ID);
        product.setNumber(PROD10001_NUMBER);
        product.setProductname(PROD10001_NAME);
        product.setPrice(PROD10001_PRICE);
        product.setAmount(PROD10001_AMOUNT);
        product.setAmountmin(PROD10001_AMOUNTMIN);
        product.setActive(PROD10001_ACTIVE);
        inventory.updateProduct(product);

        // SetUp Product 10003
        product = inventory.getProduct(PROD10003_NUMBER);
        product.setActive(true);
        product.setAmount(31);
        inventory.updateProduct(product);

        // SetUp Product 10003
        product = inventory.getProduct(PROD10004_NUMBER);
        product.setActive(true);
        product.setAmount(24);
        inventory.updateProduct(product);

    }

    /**
     * Test commitProductReservation().
     * @throws ReservationNotPossible ReservationNotPossible
     */
    @Test
    public void testCommitProductReservation() throws ReservationNotPossible {
        // current amount
        final int currAmount = inventory.getProduct(PROD10003_NUMBER).getAmount();
        final int minAmount = inventory.getProduct(PROD10003_NUMBER).getAmountmin();

        // order 7 items
        final int amountToOrder = 7;
        final int reservationId = inventory.reservateProduct(PROD10003_NUMBER, amountToOrder);
        Assert.assertTrue(reservationId > 0);

        final Date deliverDate = inventory.commitProductReservation(reservationId);
        Assert.assertNotNull(deliverDate);

        final long diff = deliverDate.getTime() - new Date().getTime();
        final int numOfDaysDiff = (int) (diff / (1000 * 60 * 60 * 24L));
        Assert.assertTrue(numOfDaysDiff >= 0); // deliverdate should be today or later

        // amount should be still the same
        if ((currAmount - amountToOrder) > minAmount) {
            Assert.assertEquals(currAmount, inventory.getProduct(PROD10003_NUMBER).getAmount() + amountToOrder);
        } else {
            Assert.assertNotEquals(currAmount, inventory.getProduct(PROD10003_NUMBER).getAmount());
        }
    }

    /**
     * Test doStockOrder().
     * @throws InvocationTargetException InvocationTargetException
     * @throws IllegalAccessException IllegalAccessException
     * @throws NoSuchMethodException NoSuchMethodException
     */
    @Test
    public void testDoStockOrder() throws IllegalAccessException, InvocationTargetException, NoSuchMethodException {
        final int orderAmount = 3;
        final int availableAmount = inventory.getProduct(PROD10003_NUMBER).getAmount();

        // execute private method doStockOrder() with reflection
        // private Date doStockOrder(final int productNumber, final int amount)
        final Method method = inventory.getClass().getDeclaredMethod("doStockOrder", int.class, int.class);
        method.setAccessible(true);
        final Date deliverDate = (Date) method.invoke(inventory, PROD10003_NUMBER, orderAmount);
        Assert.assertNotNull(deliverDate);

        // stock order has not yet delivered - amount should still the same
        Assert.assertEquals(availableAmount, inventory.getProduct(PROD10003_NUMBER).getAmount());
    }

    /**
     * Force a StockOrder in any case!
     * @throws ReservationNotPossible ReservationNotPossible
     */
    @Test
    public void testForceDoStockOrder() throws ReservationNotPossible {
        // current amount
        final int currAmount = inventory.getProduct(PROD10004_NUMBER).getAmount();
        final int minAmount = inventory.getProduct(PROD10004_NUMBER).getAmountmin();

        // order 2 * minAmount + currAmount
        final int amountToOrder = (minAmount * 2) + currAmount;
        final int reservationId = inventory.reservateProduct(PROD10004_NUMBER, amountToOrder);
        Assert.assertTrue(reservationId > 0);

        final Date deliverDate = inventory.commitProductReservation(reservationId);
        Assert.assertNotNull(deliverDate);

        final long diff = deliverDate.getTime() - new Date().getTime();
        final int numOfDaysDiff = (int) (diff / (1000 * 60 * 60 * 24L));
        Assert.assertTrue(numOfDaysDiff >= 0); // deliverdate should be today or later

        // amount should be still the same
        Assert.assertNotEquals(currAmount, inventory.getProduct(PROD10004_NUMBER).getAmount());
    }

    /**
     * Test GenerateStockProductNumber().
     */
    @Test
    public void testGenerateStockProductNumber() {
        Assert.assertEquals("000006", inventory.generateStockProductNumber(6));
        Assert.assertEquals("123456", inventory.generateStockProductNumber(123456));
        Assert.assertEquals("010101", inventory.generateStockProductNumber(10101));
        Assert.assertEquals("066660", inventory.generateStockProductNumber(66660));
    }

    /**
     * Test GetProduct().
     */
    @Test
    public void testGetProduct() {
        final Product product = inventory.getProduct(PROD10001_NUMBER);
        Assert.assertNotNull(product);

        Assert.assertEquals(PROD10001_ID, product.getId());
        Assert.assertEquals(PROD10001_NUMBER, product.getNumber());
        Assert.assertEquals(PROD10001_NAME, product.getProductname());
        Assert.assertEquals(PROD10001_ACTIVE, product.isActive());
        Assert.assertEquals(PROD10001_PRICE, product.getPrice(), 0.05);
        Assert.assertEquals(PROD10001_AMOUNT, product.getAmount());
        Assert.assertEquals(PROD10001_AMOUNTMIN, product.getAmountmin());
    }

    /**
     * Test GetProduct().
     */
    @Test
    public void testGetProducts() {
        final List<Product> products = inventory.getProducts();
        Assert.assertNotNull(products);

        Product product = null;

        // Search Product with the ProdNumber 10001
        for (final Product loopProduct : products) {
            if (loopProduct.getNumber() == PROD10001_NUMBER) {
                product = loopProduct;
                break;
            }
        }
        Assert.assertNotNull(product);

        assertEquals(PROD10001_ID, product.getId());
        assertEquals(PROD10001_NUMBER, product.getNumber());
        assertEquals(PROD10001_NAME, product.getProductname());
        assertEquals(PROD10001_ACTIVE, product.isActive());
        assertEquals(PROD10001_PRICE, product.getPrice(), 0.05);
        assertEquals(PROD10001_AMOUNT, product.getAmount());
        assertEquals(PROD10001_AMOUNTMIN, product.getAmountmin());
    }

    /**
     * Test productActive() -- True.
     */
    @Test
    public void testProductActive() {
        // should be active
        Assert.assertEquals(PROD10001_ACTIVE, inventory.productActive(PROD10001_NUMBER));
    }

    /**
     * Test productExists() -- True.
     */
    @Test
    public void testProductExists() {
        // should exists
        Assert.assertEquals(PROD10001_EXISTS, inventory.productExists(PROD10001_NUMBER));
    }

    /**
     * Test productActive() -- False.
     */
    @Test
    public void testProductNotActive() {
        // should NOT be active
        Assert.assertEquals(PROD10000_ACTIVE, inventory.productActive(PROD10000_NUMBER));
    }

    /**
     * Test productExists() -- False.
     */
    @Test
    public void testProductNotExists() {
        // should NOT exists
        Assert.assertEquals(PROD666_EXISTS, inventory.productExists(PROD666_NUMBER));
    }

    /**
     * Test Product Reservation Methods. reservateProduct() getAmountReserved() via reflections
     * updateProductReservation() deleteProductReservation()
     * @throws NoSuchMethodException No Such Method Exception
     * @throws InvocationTargetException InvocationTargetException
     * @throws IllegalAccessException IllegalAccessException
     * @throws ReservationNotExistsException ReservationNotExistsException
     * @throws ReservationNotPossible ReservationNotPossible
     */
    @Test
    public void testProductReservationMethods() throws NoSuchMethodException, IllegalAccessException,
    InvocationTargetException, ReservationNotExistsException, ReservationNotPossible {

        // verify pseudy Test-Data
        Assert.assertEquals(PROD10001_AMOUNT, inventory.getProduct(PROD10001_NUMBER).getAmount());

        // reserve only 5 pieces
        int amountToReservate = 5;

        final int reservationId = inventory.reservateProduct(PROD10001_NUMBER, amountToReservate);
        Assert.assertTrue(reservationId > 0);

        // test private method getAmountReserved() with reflection
        // InventorySystemImpl inventorySystemImpl = new InventorySystemImpl();
        // Method method = InventorySystemImpl.class.getDeclaredMethod("getAmountReserved", int.class);
        // method.setAccessible(true);
        // int amountReservedInventory = (int) method.invoke(inventorySystemImpl, reservationId);
        // Assert.assertEquals(amountToReservate, amountReservedInventory);

        // verify reservation amount
        Assert.assertEquals(PROD10001_AMOUNT - amountToReservate, inventory.getProduct(PROD10001_NUMBER).getAmount());

        // change reservation to reserve now 10 pieces
        amountToReservate = 10;
        Assert.assertTrue(inventory.updateProductReservation(reservationId, amountToReservate));

        // verify reservation amount
        Assert.assertEquals(PROD10001_AMOUNT - amountToReservate, inventory.getProduct(PROD10001_NUMBER).getAmount());

        // remove reservation
        Assert.assertTrue(inventory.deleteProductReservation(reservationId));

        // there should no more reserved items
        Assert.assertEquals(PROD10001_AMOUNT, inventory.getProduct(PROD10001_NUMBER).getAmount());
    }

    /**
     * Test ReservateProduct() that should fail.
     * @throws ReservationNotPossible ReservationNotPossible
     */
    @Test(expected = ReservationNotPossible.class)
    public void testReservateProductFail() throws ReservationNotPossible {
        // try to reserve a non existent product
        final int amountToReservate = 666;

        inventory.reservateProduct(PROD666_NUMBER, amountToReservate);
    }

    // public Date commitProductReservation(final int reservationId) {
}
