package ch.hslu.appe.usermanagement.session;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import org.junit.Before;
import org.junit.Test;

import ch.hslu.appe.usermanagement.exception.IllegalNameException;
import ch.hslu.appe.usermanagement.logic.session.Session;
import ch.hslu.appe.usermanagement.logic.session.SessionManager;
import ch.hslu.appe.usermanagement.model.User;

/**
 * Session Manager Test.
 */
public class SessionManagerTest {

    private User dummyUser;
    private SessionManager manager;


    /**
     * JUnit Setup.
     * @throws IllegalNameException Illegal Username
     */
    @Before
    public final void setUp() throws IllegalNameException {

        manager = SessionManager.getInstance();

        // Create a dummy user
        dummyUser = new User();
        dummyUser.setLogin("dummy");
    }


    /**
     * Tests the clear method.
     */
    @Test
    public final void testClearSession() {

        // Create a session, so there should be at least one valid session
        manager.createSession(dummyUser, 30);

        // Clear the sessions
        manager.clear();

        // Expected to delete all the session within the manager
        assertEquals(0, manager.count());
    }


    /**
     * Tests the count method.
     * @throws IllegalNameException Illegal Username
     */
    @Test
    public final void testCount() throws IllegalNameException {

        // First, clear all sessions
        manager.clear();

        // Create 10 sessions with individual users
        for (int index = 0; index < 10; index++) {

            final User user = new User();
            user.setLogin("User_" + index);
            manager.createSession(user, 30);
        }

        // Expected 10 active sessions
        assertEquals(10, manager.count());
    }

    /**
     * Tests if its possible to create a session with no user.
     */
    @Test
    public final void testCreateEmptySession() {

        // Create a session with an empty user
        final Session session = manager.createSession(null, 30);

        // Session is expected to be null
        assertNull(session);
    }


    /**
     * Tests the createSession method.
     */
    @Test
    public final void testCreateSession() {

        // Create a new session
        final Session session = manager.createSession(dummyUser, 30);

        // Expected to retrieve the same session as created earlier
        assertEquals(session, manager.getUserSession(dummyUser.getLogin()));
    }


    /**
     * Tests, if its possible to create a session with the same user twice.
     */
    @Test
    public final void testeCreateSessionTwice() {

        // Clear all sessions (tabula rasa :D)
        manager.clear();

        // Create a session twice with the same user
        manager.createSession(dummyUser, 30);
        manager.createSession(dummyUser, 30);

        // Expected amount of create session is just one
        assertEquals(1, manager.count());
    }

    /**
     * Tests Expired Session.
     */
    @Test
    public final void testIsInvalidByExpiredSession() {

        // Clear all previous sessions
        manager.clear();

        // Create a new session
        final Session session = manager.createSession(dummyUser, 30);

        // Decrease the session expire time by fourty minutes
        session.extend(-40);

        // Session shouldn't be valid anylonger
        assertFalse(manager.isValid(session));
    }


    /**
     * Tests invalid Session Key.
     */
    @Test
    public final void testIsInvalidByKey() {

        // Clear all previous sessions
        manager.clear();

        // Create a new session
        final Session session = manager.createSession(dummyUser, 30);

        // Session should be valid
        assertTrue(manager.isValid(session.getKey()));
    }

    /**
     * Tests invalid Session.
     */
    @Test
    public final void testIsInvalidBySession() {

        // Clear all previous sessions
        manager.clear();

        // Create a new session
        final Session session = manager.createSession(dummyUser, 30);

        // Session should be valid
        assertTrue(manager.isValid(session));
    }
}
